/*********************************************************************

  Mark.java	(C) Kenneth J. Turner, Mark A. Jennings	06/12/01 V1.3

  This class represents a course mark and its details.

  KJT 15/08/01:

    o Minor tidy-up of code

    o New "set" methods

  KJT 06/12/01:

    o Colour added

    o Mark images replaced by drawing

    o Scenario change monitored

    o Mark limits calculated

*********************************************************************/

import waba.ui.*;
import waba.fx.*;
import waba.sys.*;
import extra.util.*;

public class Mark {
  // miscellaneous constants
  private static final Color circleColour =	// two-lengths circle colour
    new Color (96, 64, 32);
  private static final Color markColour =	// mark colour
    new Color (255, 196, 32);

  private int xPos;
  private int yPos;
  Image markCircleImage;
  Image markPortImage;
  Image markStarboardImage;
  private int markDir;
  private int windDir;
  private float anglePrev;
  private int angleNext = -1;

  // normal constructor

  public Mark (int xCoord, int yCoord, int dir, int wind) {
    SailRule.scenarioChange = true;		// note scenario change
    xPos = xCoord;
    yPos = yCoord;
    markDir = dir;
    windDir = wind;
  }

  // constructor invoked when re-instantiating during file load

  public Mark (int xCoord, int yCoord, int dir, int wind, int angle) {
    this (xCoord, yCoord, dir, wind);
    angleNext = angle;
  }

  // draw mark

  public void drawMark (Graphics g) {
    g.setBackColor (markColour);
    g.fillCircle (xPos, yPos, 6);
    g.setBackColor (SailRule.mapColour);
    g.setForeColor (circleColour);
    g.drawCircle (xPos, yPos, 54);
    g.setForeColor (Color.BLACK);
    g.drawCircle (xPos, yPos, 6);
    g.drawText (markDir == Boat.PORT ? "P" : "S", xPos - 1 , yPos - 5);
    if (angleNext != -1)
      g.drawText ("" + angleNext, xPos - 6, yPos + 7);
  }

  // get heading to next mark

  public int getHeading () {
    return angleNext;
  }

  // work out which boat is taking the inside line round the mark

  public Boat getInside (Boat a, Boat b) {
    if (a.isPort (b))
      return (markDir == Boat.STARBOARD ? a : b);
    else
      return (markDir == Boat.STARBOARD ? b : a);
  }

  // return the mark corners enlarged by 1 unit;
  // first index 0 = back left, 1 = back right, 2 = front right,
  // 3 = front left (i.e. anticlockwise); second index is 0 = X, 1 = Y

  public int[][] getLimits () {
    int limits[][] = new int[4][2];
    limits[0][0] = xPos - 7;
    limits[0][1] = yPos + 7;
    limits[1][0] = xPos + 7;
    limits[1][1] = yPos + 7;
    limits[2][0] = xPos + 7;
    limits[2][1] = yPos - 7;
    limits[3][0] = xPos - 7;
    limits[3][1] = yPos - 7;
    return (limits);			// return mark limits
  }

  // get mark rounding tack

  public int getTack () {
    return (markDir);
  }

  // get X coordinate

  public int getX () {
    return xPos;
  }

  // get Y coordinate

  public int getY () {
    return yPos;
  }

  // given x and y coordinates, calculate what angle they make relative to the
  // mark position

  public void indicateNext (int x, int y) {
    int xDist = x - xPos;
    int yDist = yPos - y;
    float nextAngle = (Maths.atan2 (xDist, yDist) / (2 * Maths.PI)) * 360;
    if (nextAngle < 0)
      nextAngle = 360 + nextAngle;
    angleNext = (int) (nextAngle);
  }

  // check if boat on given heading must tack to round the mark

  public boolean mustTack (Boat boat) {
    int oldDiff = (boat.getHeading () - windDir) % 360;
    if (oldDiff < 0)
      oldDiff += 360;
    int newDiff = (angleNext - windDir) % 360;
    if (newDiff < 0)
      newDiff += 360;

      return (
      (oldDiff < 180 && newDiff > 180) ||
        (oldDiff > 180 && newDiff < 180));
  }

  // check if a boat is on a beat to the mark (i.e. angle to mark is less
  // than 90 degrees)

  public boolean onBeat (Boat boat) {
    int xDist = xPos - boat.getX ();
    int yDist = yPos - boat.getY ();
    float markAngle = 90f + Maths.atan2 (yDist, xDist) * 180f / Maths.PI;
    int angleDiff = ((int) markAngle - windDir) % 360;
    if (angleDiff < 0)
      angleDiff += 360;
    return (angleDiff < 90 || angleDiff > 270);
  }

  // set mark coordinates

  public void setCoordinates (int x, int y) {
    SailRule.scenarioChange = true;		// note scenario change
    xPos = x;
    yPos = y;
  }

  // set next mark heading

  public void setHeading (int angle) {
    SailRule.scenarioChange = true;		// note scenario change
    angleNext = angle;
  }

  // set mark tack

  public void setTack (int dir) {
    markDir = dir;
  }
}
